@@ -12,8 +12,8 @@ from . import io
 from .io import *
 from . import processing
 from .processing import *
-from . import viz
-from .viz import plot_utils, jupyter_utils
+from sidpy import viz
+from sidpy.viz import plot_utils, jupyter_utils
 from .__version__ import version as __version__
 
 __all__ = ['__version__']
