@@ -20,15 +20,14 @@ import matplotlib.pyplot as plt
 from sidpy.base.string_utils import validate_single_string_arg, \
     validate_list_of_strings
 from sidpy.base.num_utils import contains_integers, get_exponent
-from sidpy.hdf.hdf_utils import get_attr, lazy_load_array
+from sidpy.hdf.hdf_utils import get_attr, lazy_load_array, copy_attributes
 from sidpy.hdf.dtype_utils import flatten_to_real, is_complex_dtype
 from sidpy.viz.jupyter_utils import simple_ndim_visualizer
 from sidpy.viz.plot_utils import plot_map, get_plot_grid_size
 
 from .hdf_utils import check_if_main, create_results_group, write_reduced_anc_dsets, link_as_main, \
-    get_dimensionality, get_sort_order, get_unit_values, reshape_to_n_dims, write_main_dataset, reshape_from_n_dims, \
-    copy_attributes
-from .write_utils import Dimension
+    get_dimensionality, get_sort_order, get_unit_values, reshape_to_n_dims, write_main_dataset, reshape_from_n_dims
+from .dimension import Dimension
 
 if sys.version_info.major == 3:
     unicode = str
@@ -489,8 +488,8 @@ class USIDataset(h5py.Dataset):
         if verbose:
             print('data_slice of shape: {} after squeezing'.format(data_slice.shape))
 
-        pos_inds = self.h5_pos_inds[pos_slice, :]
-        spec_inds = self.h5_spec_inds[:, spec_slice].reshape([self.h5_spec_inds.shape[0], -1])
+        pos_inds = self.h5_pos_inds[pos_slice.ravel(), :]
+        spec_inds = self.h5_spec_inds[:, spec_slice.ravel()].reshape([self.h5_spec_inds.shape[0], -1])
         if verbose:
             print('Sliced position indices:')
             print(pos_inds)
@@ -920,7 +919,7 @@ class USIDataset(h5py.Dataset):
                 fig, axes = plt.subplots(nrows=2, **fig_args)
                 for axis, ufunc, comp_name in zip(axes.flat, [np.abs, np.angle], ['Magnitude', 'Phase']):
                     axis.plot(ref_dims[0].values, ufunc(np.squeeze(curve)), **kwargs)
-                    if comp_name is 'Magnitude':
+                    if comp_name == 'Magnitude':
                         axis.set_title(self.name + '\n(' + comp_name + ')', pad=15)
                         axis.set_ylabel(self.data_descriptor)
                     else:
@@ -971,7 +970,7 @@ class USIDataset(h5py.Dataset):
                 fig, axes = plt.subplots(nrows=2, **fig_args)
                 for axis, ufunc, comp_name in zip(axes.flat, [np.abs, np.angle], ['Magnitude', 'Phase']):
                     cbar_label = self.data_descriptor
-                    if comp_name is 'Phase':
+                    if comp_name == 'Phase':
                         cbar_label = 'Phase (rad)'
                     plot_map(axis, ufunc(np.squeeze(img)), show_xy_ticks=True, show_cbar=True,
                              cbar_label=cbar_label, x_vec=ref_dims[1].values, y_vec=ref_dims[0].values,
@@ -1255,7 +1254,7 @@ class USIDataset(h5py.Dataset):
 
         """
         Create the spectral and position labels for the dataset in string form then
-        create the position value array in string form, right-strip the last comma from the 
+        create the position value array in string form, right-strip the last comma from the
         string to deliver the correct number of values, append all of the labels and values together,
         save the data and header to a temporary csv output
         """
