@@ -13,16 +13,19 @@ import numpy as np
 import psutil
 import time as tm
 import h5py
+from warnings import warn
 from numbers import Number
 from multiprocessing import cpu_count
 
-from .comp_utils import parallel_compute, get_MPI, group_ranks_by_socket, \
-    get_available_memory
-from ..io.hdf_utils import check_if_main, check_for_old, get_attributes
+from sidpy.proc.comp_utils import parallel_compute, get_MPI, \
+    group_ranks_by_socket, get_available_memory
+from sidpy.base.num_utils import integers_to_slices
+from sidpy.base.string_utils import validate_single_string_arg, format_time, \
+    format_size
+from sidpy.hdf.hdf_utils import write_simple_attrs, lazy_load_array
+
+from ..io.hdf_utils import check_if_main, check_for_old
 from ..io.usi_data import USIDataset
-from ..io.dtype_utils import integers_to_slices, lazy_load_array, \
-    validate_single_string_arg
-from ..io.io_utils import format_time, format_size
 
 # TODO: internalize as many attributes as possible. Expose only those that will be required by the user
 
@@ -42,7 +45,7 @@ class Process(object):
         ----------
         h5_main : :class:`~pyUSID.io.usi_data.USIDataset`
             The USID main HDF5 dataset over which the analysis will be performed.
-         self.process_name : str
+        process_name : str
             Name of the process
         cores : uint, optional
             How many cores to use for the computation. Default: all available cores - 2 if operating outside MPI context
@@ -142,6 +145,13 @@ class Process(object):
 
         MPI = get_MPI()
 
+        # Ensure that the file is opened in the correct comm or something
+        if MPI is not None and h5_main.file.driver != 'mpio':
+            warn('Code was called in MPI context but HDF5 file was not opened '
+                 'with the "mpio" driver. JobLib will be used instead of MPI '
+                 'for parallel computation')
+            MPI = None
+
         if MPI is not None:
             # If we came here then, the user has intentionally asked for multi-node computation
             comm = MPI.COMM_WORLD
@@ -161,11 +171,6 @@ class Process(object):
             if self.mpi_rank == 0:
                 print('Working on {} ranks via MPI'.format(self.mpi_size))
 
-            # Ensure that the file is opened in the correct comm or something
-            if h5_main.file.driver != 'mpio':
-                raise TypeError('The HDF5 file should have been opened with driver="mpio". Current driver = "{}"'
-                                ''.format(h5_main.file.driver))
-
             if verbose and self.mpi_rank == 0:
                 print('Finished getting all necessary MPI information')
 
@@ -276,7 +281,7 @@ class Process(object):
 
         if self.mpi_rank == 0:
             print('Consider calling test() to check results before calling compute() which computes on the entire'
-                  ' dataset and writes back to the HDF5 file')
+                  ' dataset and writes results to the HDF5 file')
 
         self.duplicate_h5_groups, self.partial_h5_groups = self._check_for_duplicates()
 
@@ -771,6 +776,17 @@ class Process(object):
                 if completed_pixels > 0:
                     self._h5_status_dset[:completed_pixels] = 1
 
+    def _write_source_dset_provenance(self):
+        """
+        Writes path of HDF5 file and path of h5_main to the results group
+        if results are being written to a new HDF5 file
+        """
+        if self.h5_main.file == self.h5_results_grp.file:
+            return
+        write_simple_attrs(self.h5_results_grp,
+                           {'source_file_path': self.h5_main.file.filename,
+                            'source_dataset_path': self.h5_main.name})
+
     def _get_existing_datasets(self):
         """
         The purpose of this function is to allow processes to resume from partly computed results
@@ -892,6 +908,7 @@ class Process(object):
             if self.verbose and self.mpi_rank == 0:
                 print('Creating HDF5 group and datasets to hold results')
             self._create_results_datasets()
+            self._write_source_dset_provenance()
         else:
             # resuming from previous checkpoint
             resuming = True
