@@ -11,8 +11,12 @@ Submodules
     jupyter_utils
 
 """
+from warnings import warn
 
-from . import plot_utils
-from . import jupyter_utils
+warn('Please use sidpy.viz.plot_utils instead of pyUSID.viz.plot_utils. '
+     'pyUSID.plot_utils will be removed in a future release of pyUSID',
+     FutureWarning)
+
+from sidpy.viz import plot_utils, jupyter_utils
 
 __all__ = ['plot_utils', 'jupyter_utils']
